#include <FK/FK.h>
#include "TrackBall.h"
#include "CollisionBox.h"
#include "FrameController.h"

int main(int argc, char *argv[])
{
	Fl_Window		mainWindow(800, 600, "COLLISION TEST");
	fk_Model		camera, lightModel, sphereModel;
	fk_Light		light;
	fk_Sphere		sphere(4, 2.0);
	fk_Scene		scene;
	fk_Window		window(0, 0, 800, 600);

	fk_TrackBall	tb(&window, &camera);
	fk_Vector		spd, acc, view, norm, back, tmpBack;
	fk_Matrix		rotMat;

	CollisionBox	box[7];

	fkut_FrameController	fc;

	mainWindow.end();
	fk_InitMaterial();

	// Ɩ̐ݒ
	lightModel.setShape(&light);
	lightModel.setMaterial(White);
	lightModel.glMoveTo(0.0, 0.0, 0.0);
	lightModel.glFocus(0.5, -1.0, 0.0);

	// ̂̐ݒ
	box[0].setSize(100.0, 1.0, 100.0);
	box[0].model.setMaterial(Yellow);

	box[1].setSize(20.0, 10.0, 10.0);
	box[1].setPos(fk_Vector(20.0, 0.0, 10.0));
	box[1].model.setMaterial(Green);

	box[2].setSize(10.0, 50.0, 10.0);
	box[2].setPos(fk_Vector(30.0, 0.0, -30.0));
	box[2].model.setMaterial(Green);
	box[2].model.glRotateWithVec(box[2].pos, fk_X, FK_PI/2.5);

	box[3].setSize(30.0, 30.0, 10.0);
	box[3].setPos(fk_Vector(-20.0, 0.0, -10.0));
	box[3].model.setMaterial(Green);
	box[3].model.glRotateWithVec(box[3].pos, fk_Y, FK_PI/2.5);

	box[4].setSize(20.0, 10.0, 40.0);
	box[4].setPos(fk_Vector(-20.0, 0.0, 10.0));
	box[4].model.setMaterial(Green);
	box[4].model.glRotateWithVec(box[4].pos, fk_Z, FK_PI/2.5);

	box[5].setSize(10.0, 10.0, 10.0);
	box[5].setPos(fk_Vector(0.0, 0.0, 0.0));
	box[5].model.setMaterial(Green);

	box[6].setSize(10.0, 10.0, 10.0);
	box[6].setPos(fk_Vector(7.5, 0.0, 0.0));
	box[6].model.setMaterial(Green);

	// ̂̐ݒ
	sphereModel.setShape(&sphere);
	sphereModel.setMaterial(Cyan);
	sphereModel.setSmoothMode(true);
	sphereModel.glMoveTo(0.0, 15.0, 0.0);

	// effBXvCXgɓo^
	scene.entryCamera(&camera);
	for(int i = 0; i < 7; i++) {
		box[i].model.setSmoothMode(true);
		scene.entryModel(&box[i].model);
	}
	scene.entryModel(&lightModel);
	scene.entryModel(&sphereModel);

	// EBhEփfBXvCXgo^
	window.setScene(&scene);

	// _̈ʒuƎpݒ
	camera.glMoveTo(0.0, 100.0, 100.0);
	camera.glFocus(0.0, 0.0, 0.0);
	camera.glUpvec(0.0, 1.0, 0.0);

	rotMat.makeRot(FK_PI/2.0, fk_Y);

	mainWindow.show();
	window.show();

	while(true) {
		static bool prevJump = false;

		fc.TimeRegular();

		if(mainWindow.visible() == 0) {
			if(Fl::wait() == 0) {
				break;
			} else {
				continue;
			}
		}

		tb.update();

		if(fc.GetDrawFlag()) window.drawWindow();
		if(Fl::check() == 0) break;
		if(window.winOpenStatus() == false) continue;

		// J XZ Ƃɐis
		view = camera.getVec();
		view.y = 0.0;
		view.normalize();

		// Wv̏x XZ ̊x悹
		if(window.getKeyStatus(' ', false) & !prevJump) {
			acc.x = spd.x;
			acc.z = spd.z;
			acc.y = 2.0;
		}
		prevJump = window.getKeyStatus(' ', false);

		// n㎞͓x^A󒆂ł͉x
		spd.init();
		if(window.getSpecialKeyStatus(FK_UP, false)) {
			if(abs(acc.y) > 0.0) acc += view*0.1;
			else spd += view*0.5;
		}
		if(window.getSpecialKeyStatus(FK_DOWN, false)) {
			if(abs(acc.y) > 0.0) acc -= view*0.1;
			else spd -= view*0.5;
		}
		if(window.getSpecialKeyStatus(FK_LEFT, false)) {
			if(abs(acc.y) > 0.0) acc += rotMat*view*0.1;
			else spd += rotMat*view*0.5;
		}
		if(window.getSpecialKeyStatus(FK_RIGHT, false)) {
			if(abs(acc.y) > 0.0) acc -= rotMat*view*0.1;
			else spd -= rotMat*view*0.5;
		}
		if(spd.normalize()) spd *= 0.5;

		// d͉x
		acc.y -= 0.098;
		if(acc.x*acc.x + acc.z*acc.z > 0.5*0.5) {
			double tmpSpdLen = sqrt(acc.x*acc.x + acc.z*acc.z);
			acc.x *= 0.5/tmpSpdLen;
			acc.z *= 0.5/tmpSpdLen;
		} else {
			acc.x *= 0.9;
			acc.z *= 0.9;
		}

		// x𑬓xɏ悹
		spd += acc;

		// Փ˔̊Jn
		for(int i = 0; i < 7; i++) {
			if(i == 0) {
				box[i].model.setMaterial(Yellow);
			} else {
				box[i].model.setMaterial(Green);
			}
		}
		do {
			back.init();
			for(int i = 0; i < 7; i++) {
				tmpBack =  box[i].detectCollision(sphereModel.getPosition(), spd+back);
				if(!tmpBack.isZero()) {
					back += tmpBack;
					box[i].model.setMaterial(Red);
				}
			}
			if(abs(back.y) > FK_EPS) {
				acc.init();
			}
			spd += back;
		} while(!back.isZero());
		// Փ˂̔͂𑬓xɉďI

		// x𑀍샂fɉĎۂɓ
		sphereModel.glTranslate(spd);
	}

	return 0;
}
